#include "fluoroethene.h"

typedef struct
{
  Kenshi *c1, *c2;
  int turn;
} game_state;


/* 外部変数 */
extern const Sword SwordList[SWORD_VARIATION];

/* 死亡判定 */
static int dead_check(game_state *gs){
        Kenshi *c1 = gs->c1, *c2 = gs->c2;
        if (c1->heavy_push > 0) {
          if (c1->heavy_push > c2->heavy_push)
            return C1_WIN;
          else if(c1->heavy_push < c2->heavy_push)
            return C2_WIN;
          else
            return DRAW;
        } else if (c2->heavy_push > 0) {
          return C2_WIN;
        }
                  
	if (c1->hp <= 0) {
		if (c1->life > 0) {
			c1->life--;
			c1->hp = 1;
			if (c2->hp <= 0) {
				if (c2->life > 0) {
					c2->life--;
					c2->hp = 1;
					return CONTINUE;
				} else {
					return C1_WIN;
				}
			} else {
				return CONTINUE;
			}
		} else {
			if (c2->hp <= 0) {
				if (c2->life > 0) {
					c2->life--;
					c2->hp = 1;
					return C2_WIN;
				} else if (c1->hp > c2->hp) {
					return C1_WIN;
				} else if (c1->hp < c2->hp) {
					return C2_WIN;
				} else {
					return DRAW;
				}
			} else {
				return C2_WIN;
			}
		}
	} else if (c2->hp <= 0) {
		if (c2->life > 0) {
			c2->life--;
			c2->hp = 1;
			return CONTINUE;
		} else {
			return C1_WIN;
		}
	} else {
		return CONTINUE;
	}
}

/* 一回単独行動 */
static void move(Kenshi *mover, Kenshi *opponent){
	int i, sa, flg;

	mover->ap--;
	if (mover->stun > 0) {
		mover->stun--;
		return;
	}
	for (i=0; 1; i++) {
		if (mover->swords[i] == -1) {
			return;
		} else if (mover->state[i] == SS_ACTIVE) {
			mover->state[i] = (SwordList[mover->swords[i]].limited ? SS_LOST : SS_USED);
			break;
		}
	}
	if (SwordList[mover->swords[i]].category == SC_ATTACK) {
		sa = 0;
		flg = 1;
		while (flg) {
			flg = 0;
			if (sa) {
				if (mover->reflection > 0) {
					mover->reflection--;
					sa = !sa;
					flg = 1;
				}
			} else {
				if (opponent->reflection > 0) {
					opponent->reflection--;
					sa = !sa;
					flg = 1;
				}
			}
		}
		if (sa) {
			SwordList[mover->swords[i]].body(mover, mover);
		} else {
			SwordList[mover->swords[i]].body(mover, opponent);
		}
	} else {
		SwordList[mover->swords[i]].body(mover, mover);
	}
}

/* 1ターン戦闘進行 */
static int gain(game_state *gs){
	int i, j, sa1, sa2, flg1, flg2, pre, _flg1;
        Kenshi *c1 = gs->c1, *c2 = gs->c2;

# define CHECK if(gs->turn > 0) { int ret = dead_check(gs); if(ret) return ret; }

	/* ターン開始フェイズ */
        CHECK

	/* 状態効果フェイズ */
	if (c1->poison > 0) {
		c1->hp -= c1->poison;
	}
	if (c2->poison > 0) {
		c2->hp -= c2->poison;
	}
        CHECK

	/* 行動回数セットフェイズ */
	ken_set_ap(c1, gs->turn);
	ken_set_ap(c2, gs->turn);

	/* 行動フェイズ */
	if (c1->speed > c2->speed) {
		while (c1->ap > 0) {
			move(c1, c2);
                        CHECK
		}
		while (c2->ap > 0) {
			move(c2, c1);
                        CHECK
		}
	} else if (c1->speed < c2->speed) {
		while (c2->ap > 0) {
			move(c2, c1);
                        CHECK
		}
		while (c1->ap > 0) {
			move(c1, c2);
                        CHECK
		}
	} else {
		while (c1->ap>0 || c2->ap>0) {
			/* 途中で素早さが変わっていたら別行動 */
			if (c1->speed > c2->speed) {
				while (c1->ap > 0) {
					move(c1, c2);
                                        CHECK
				}
				while (c2->ap > 0) {
					move(c2, c1);
                                        CHECK
				}
			} else if (c1->speed < c2->speed) {
				while (c2->ap > 0) {
					move(c2, c1);
                                        CHECK
				}
				while (c1->ap > 0) {
					move(c1, c2);
                                        CHECK
				}
			/* 一方のAPがなくなったら単独行動 */
			} else if (c1->ap <= 0) {
				while (c2->ap > 0) {
					move(c2, c1);
                                        CHECK
				}
			} else if (c2->ap <= 0) {
				while (c1->ap > 0) {
					move(c1, c2);
                                        CHECK
				}
			/* いずれかがスタンしていれば一回単独行動 */
			} else if (c1->stun > 0) {
				move(c1, c2);
				move(c2, c1);
                                CHECK
			} else if (c2->stun > 0) {
				move(c2, c1);
				move(c1, c2);
                                CHECK
			/* どちらもスタンがない場合 */
			} else {
				for (i=0; 1; i++) {
					if (c1->swords[i] == -1) {
						i = -1;
						break;
					} else if (c1->state[i] == SS_ACTIVE) {
						break;
					}
				}
				for (j=0; 1; j++) {
					if (c2->swords[j] == -1) {
						j = -1;
						break;
					} else if (c2->state[j] == SS_ACTIVE) {
						break;
					}
				}
				/* 一方が剣を使いつくしたなら一回単独行動 */
				if (i == -1) {
					c1->ap--;
					move(c2, c1);
                                        CHECK
				} else if (j == -1) {
					c2->ap--;
					move(c1, c2);
                                        CHECK
				/* 一方が補助なら一回単独行動 */
				} else if (SwordList[c1->swords[i]].category == SC_STATUS) {
					move(c1, c2);
					move(c2, c1);
                                        CHECK
				} else if (SwordList[c2->swords[j]].category == SC_STATUS) {
					move(c2, c1);
					move(c1, c2);
                                        CHECK
				/* 両方が攻撃なら一回超同時 */
				} else {
					c1->state[i] = (SwordList[c1->swords[i]].limited ? SS_LOST : SS_USED);
					c2->state[j] = (SwordList[c2->swords[j]].limited ? SS_LOST : SS_USED);
					sa1 = 0;
					sa2 = 0;
					flg1 = 1;
					flg2 = 1;
					pre = 0;
					while (flg1 || flg2) {
						_flg1 = flg1;
						if (flg1) {
							flg1 = 0;
							if (sa1) {
								if (c1->reflection > 0) {
									c1->reflection--;
									sa1 = !sa1;
									flg1 = 1;
								}
							} else {
								if (c2->reflection > 0) {
									c2->reflection--;
									sa1 = !sa1;
									flg1 = 1;
								}
							}
							if (!flg2) {
								pre = 2;
								if (sa2) {
									SwordList[c2->swords[j]].body(c2, c2);
								} else {
									SwordList[c2->swords[j]].body(c2, c1);
								}
								dead_check(gs);
							}
						}
						if (flg2) {
							flg2 = 0;
							if (sa2) {
								if (c2->reflection > 0) {
									c2->reflection--;
									sa2 = !sa2;
									flg2 = 1;
								}
							} else {
								if (c1->reflection > 0) {
									c1->reflection--;
									sa2 = !sa2;
									flg2 = 1;
								}
							}
							if (!_flg1) {
								pre = 1;
								if (sa1) {
									SwordList[c1->swords[i]].body(c1, c1);
								} else {
									SwordList[c1->swords[i]].body(c1, c2);
								}
								dead_check(gs);
							}
						}
					}
					if (pre == 1) {
						if (sa2) {
							SwordList[c2->swords[j]].body(c2, c2);
						} else {
							SwordList[c2->swords[j]].body(c2, c1);
						}
					} else if (pre == 2) {
						if (sa1) {
							SwordList[c1->swords[i]].body(c1, c1);
						} else {
							SwordList[c1->swords[i]].body(c1, c2);
						}
					} else {
						if (sa1) {
							SwordList[c1->swords[i]].body(c1, c1);
							SwordList[c2->swords[j]].body(c2, c2);
						} else {
							SwordList[c1->swords[i]].body(c1, c2);
							SwordList[c2->swords[j]].body(c2, c1);
						}
					}
					c1->ap--;
					c2->ap--;
                                        CHECK
                               }
			}
		}
	}

	/* ターン終了フェイズ */
	gs->turn++;
	if (gs->turn > 20) {
		if (c1->hp > c2->hp) {
			return C1_WIN;
		} else if (c1->hp < c2->hp) {
			return C2_WIN;
		} else {
			return DRAW;
		}
	}
	return CONTINUE;
# undef CHECK
}

/* 戦闘を最後まで進める */
int bd_go(Kenshi *c1_org, Kenshi *c2_org){
	int ret;
        game_state gs;

	gs.c1 = c1_org;
	gs.c2 = c2_org;
	gs.turn = 0;
	for(ret=CONTINUE; ret==CONTINUE; ret=gain(&gs));
	return ret;
}

/* 剣師のコピーを取って戦闘処理する */
int bd_go_by_cpy(const Kenshi *c1_org, const Kenshi *c2_org){
	Kenshi c1_real, c2_real;

	ken_cpy(c1_org, &c1_real);
	ken_cpy(c2_org, &c2_real);
        return bd_go(&c1_real, &c2_real);
}
