#include "fluoroethene.h"
# include <stddef.h>
# include <string.h>

/* 静的関数プロトタイプ宣言 */
static int streq(const char *str0, const char *str1);
static int filter(int dam, Kenshi *target);
static void slash(Kenshi *controller, Kenshi *target);
static void heavy(Kenshi *controller, Kenshi *target);
static void shock(Kenshi *controller, Kenshi *target);
static void poison(Kenshi *controller, Kenshi *target);
static void absolute(Kenshi *controller, Kenshi *target);
static void magic(Kenshi *controller, Kenshi *target);
static void death(Kenshi *controller, Kenshi *target);
static void heat(Kenshi *controller, Kenshi *target);
static void protect(Kenshi *controller, Kenshi *target);
static void accelerate(Kenshi *controller, Kenshi *target);
static void shield(Kenshi *controller, Kenshi *target);
static void mirror(Kenshi *controller, Kenshi *target);
static void life(Kenshi *controller, Kenshi *target);
static void recover(Kenshi *controller, Kenshi *target);
static void victory(Kenshi *controller, Kenshi *target);

/* 剣を定義し，配列として保持。他ファイルから参照する。 */
const Sword SwordList[SWORD_VARIATION] = {
        {"斬", SC_ATTACK, 1, 0, slash},
        {"高", SC_ATTACK, 0, 1, slash},
        {"重", SC_ATTACK, 4, 0, heavy},
        {"衝", SC_ATTACK, 2, 0, shock},
        {"毒", SC_ATTACK, 1, 1, poison},
        {"絶", SC_ATTACK, 3, 0, absolute},
        {"魔", SC_ATTACK, 2, 1, magic},
        {"死", SC_ATTACK, 6, 1, death},
        {"熱", SC_STATUS, 1, 0, heat},
        {"護", SC_STATUS, 1, 0, protect},
        {"速", SC_STATUS, 1, 0, accelerate},
        {"盾", SC_STATUS, 1, 0, shield},
        {"鏡", SC_STATUS, 2, 1, mirror},
        {"命", SC_STATUS, 1, 1, life},
        {"回1", SC_STATUS, 1, 1, recover},
        {"回2", SC_STATUS, 2, 1, recover},
        {"回3", SC_STATUS, 3, 1, recover},
        {"回4", SC_STATUS, 4, 1, recover},
        {"回5", SC_STATUS, 5, 1, recover},
        {"回6", SC_STATUS, 6, 1, recover},
        {"回7", SC_STATUS, 7, 1, recover},
        {"回8", SC_STATUS, 8, 1, recover},
        {"回9", SC_STATUS, 9, 1, recover},
        {"回10", SC_STATUS, 10, 1, recover},
        {"回11", SC_STATUS, 11, 1, recover},
        {"回12", SC_STATUS, 12, 1, recover},
        {"回13", SC_STATUS, 13, 1, recover},
        {"回14", SC_STATUS, 14, 1, recover},
        {"回15", SC_STATUS, 15, 1, recover},
        {"回16", SC_STATUS, 16, 1, recover},
        {"回17", SC_STATUS, 17, 1, recover},
        {"回18", SC_STATUS, 18, 1, recover},
        {"回19", SC_STATUS, 19, 1, recover},
        {"回20", SC_STATUS, 20, 1, recover},
        {"回21", SC_STATUS, 21, 1, recover},
        {"勝1", SC_STATUS, 1, 1, victory},
        {"勝2", SC_STATUS, 2, 1, victory},
        {"勝3", SC_STATUS, 3, 1, victory},
        {"勝4", SC_STATUS, 4, 1, victory},
        {"勝5", SC_STATUS, 5, 1, victory},
        {"勝6", SC_STATUS, 6, 1, victory},
        {"勝7", SC_STATUS, 7, 1, victory},
        {"勝8", SC_STATUS, 8, 1, victory},
        {"勝9", SC_STATUS, 9, 1, victory},
        {"勝10", SC_STATUS, 10, 1, victory},
        {"勝11", SC_STATUS, 11, 1, victory},
        {"勝12", SC_STATUS, 12, 1, victory},
        {"勝13", SC_STATUS, 13, 1, victory},
        {"勝14", SC_STATUS, 14, 1, victory},
        {"勝15", SC_STATUS, 15, 1, victory},
        {"勝16", SC_STATUS, 16, 1, victory},
        {"勝17", SC_STATUS, 17, 1, victory},
        {"勝18", SC_STATUS, 18, 1, victory},
        {"勝19", SC_STATUS, 19, 1, victory},
        {"勝20", SC_STATUS, 20, 1, victory},
        {"勝21", SC_STATUS, 21, 1, victory}
};

/* 名前から剣番号取得。読んだ分*strを進める */
int read_sword(const char **str){
        int ret, i;
        int maxlen = -1;

        for (i=0; i<SWORD_VARIATION; i++) {
                const char *sname = SwordList[i].name;
                int len = strlen(sname);
                if (len > maxlen && strncmp(sname, *str, len) == 0)
                {
                        maxlen = len;
                        ret = i;
                }
        }
        if(maxlen == -1)
                return -1;
        *str += maxlen;
        return ret;
}

/* ガード&ダメージ判定 */
static int filter(int dam, Kenshi *target){
        if (target->guard > 0) {
                target->guard--;
                dam = (dam>>1) + (dam&1);
        }
        if (dam >= 1) {
                target->hp -= dam;
        }
        return dam;
}

/* 以下，剣の実行部分の定義 */
static void slash(Kenshi *controller, Kenshi *target){
        filter(controller->attack - target->defense, target);
}
static void heavy(Kenshi *controller, Kenshi *target){
        filter(controller->attack*2 - target->defense, target);
}
static void shock(Kenshi *controller, Kenshi *target){
        if (filter(controller->attack - target->defense, target)>=1) {
                target->stun++;
        }
}
static void poison(Kenshi *controller, Kenshi *target){
        if (filter(controller->attack - target->defense, target)>=1) {
                target->poison++;
        }
}
static void absolute(Kenshi *controller, Kenshi *target){
        filter(controller->attack, target);
}
static void magic(Kenshi *controller, Kenshi *target){
        filter((target->hp>>1) + (target->hp&1), target);
}
static void death(Kenshi *controller, Kenshi *target){
        filter(target->hp, target);
}
static void heat(Kenshi *controller, Kenshi *target){
        target->attack++;
}
static void protect(Kenshi *controller, Kenshi *target){
        target->defense++;
}
static void accelerate(Kenshi *controller, Kenshi *target){
        target->speed++;
}
static void shield(Kenshi *controller, Kenshi *target){
        target->guard++;
}
static void mirror(Kenshi *controller, Kenshi *target){
        target->reflection++;
}
static void life(Kenshi *controller, Kenshi *target){
        target->life++;
}
static void recover(Kenshi *controller, Kenshi *target){
        target->hp = target->max_hp;
}
static void victory(Kenshi *controller, Kenshi *target){
        target->heavy_push++;
}
