#include <stdio.h>
#include <stdlib.h>
#include "search.h"

static int max(int x, int y) { return x < y ? y : x; }
static int min(int x, int y) { return x < y ? x : y; }

void print_kenshi(const Kenshi *k)
{
  char buf[1000];
  ken_to_theki(k, buf);
  puts(buf);
}

typedef struct
{
  int num;
  const Kenshi *ks;
} kenshi_set;

/* 剣師集合setを使ってobjを試濁する。濁性を証明できたかどうかを返す */
int test_for_muddiness(const Kenshi *obj, const kenshi_set *set)
{
  int i;
  for(i = 0; i < set->num; i++)
    if(bd_go_by_cpy(obj, &set->ks[i]) == C2_WIN)
      return 1;
  return 0;
}

int g_counter;

/* 素早さspdで剣swordsを持ち、残りボーナスがbonus点であるマーガレットの候補を全て表示する */
void search_with_swords(int spd, int has_hp, const int *swords, int n_sword, int bonus, void *param)
{
  int has_attack_independent, has_attack_dependent;
  int i;
  int sword_count[SWORD_VARIATION] = {0};
  int attack, max_attack;

  /*
  printf("%d/", spd);
  for(i = 0; i < n_sword; i++)
    printf("%s", SwordList[swords[i]].name);
  printf("\n");
  return;
  /**/

  for(i = 0; i < n_sword; i++)
    sword_count[swords[i]]++;
  has_attack_independent = sword_count[S_MAGICAL] || sword_count[S_DEATH];
  has_attack_dependent = sword_count[S_SLASH] || sword_count[S_RAPID] || sword_count[S_HEAVY] ||
    sword_count[S_SHOCK] || sword_count[S_POISON] | sword_count[S_ABSOLUTE];
  max_attack = has_attack_dependent ? bonus : 0; /* 純正オランダの排除 */

  for(attack = 0; attack <= max_attack; attack++)
  {
    int hp_max = bonus-attack, hp_min = 1;
    int hp;
    if(!has_hp)
      hp_max = hp_min = 0;
    for(hp = hp_min; hp <= hp_max; hp++)
    {
      Kenshi x;
      int defense = bonus-attack-hp;
      int z = 5*hp + 2*defense - 12; /* 0/6/0/2/高高/の攻撃を受けた後のHP */
      if(z < 0 && z < - attack * min(spd, sword_count[S_RAPID]))
        continue; /* 0/6/0/2/高高/の攻撃に耐えられず、判定でも負ける */
      if(hp < 10 && !has_attack_independent && (!has_attack_dependent || attack == 0 && !sword_count[S_HEAT]))
        continue; /* 攻撃剣がないか、あってもヌル剣のみなら50/0/0/0//に負ける */
      if((!sword_count[S_POISON] || attack == 0 && (spd == 1 || !sword_count[S_HEAT])) /* ヌルでない蟲毒剣を1ターン目に振れず */
          && (attack + sword_count[S_HEAT]) * (sword_count[S_RAPID] + sword_count[S_SLASH]) < 5 /* 2ターン目開始時までに5ダメージを与えられず */
          && defense + min(spd, sword_count[S_PROTECTION]) < 4 /* 加護を併せても2ターン目開始時の防御力が4に満たず */
          && (spd <= 1 || spd == 2 && !sword_count[S_ACCEL])) /* 加速を併せても2ターン目開始時の素早さが2以下ならば */
        continue; /* 5/0/0/4/速熱熱衝衝/に自明に封殺される */

      g_counter++;
      ken_init(&x, 5*hp, attack, defense, spd, swords);
      if(!test_for_muddiness(&x, (const kenshi_set *)param))
        print_kenshi(&x);
    }
  }
}

/* ボーナスbonusのマーガレット候補を全て表示する */
void search_with_bonus(int bonus, const kenshi_set *set)
{
  int has_hp;
  int brem = bonus-3; /* hp+atk+defが3以上ないと0/6/0/2/高高/に負ける */
  printf("bonus: %d\n", bonus);
  for(has_hp = 0; has_hp <= 1; has_hp++)
  {
    int spd;
    for(spd = 0; spd <= brem; spd++)
      each_sword_array(spd, has_hp, brem-spd-(spd==1), bonus-spd, all_swords, search_with_swords, (void *)set);
    /* spdが1の場合、hp+atk+defが4以上ないと0/6/0/2/高高/か5/0/0/4/速熱熱衝衝/に負ける */
  }
}

void enum_all(int spd, int has_hp, const int *swords, int n_sword, int bonus, void *param)
{
  int i;
  int attack;
  int need_atk = 0;
  for(i = 0; i < n_sword; i++)
    if(swords[i] <= S_ABSOLUTE)
      need_atk = 1;
  for(attack = 0; attack <= bonus; attack++)
  {
    int hp_max = bonus-attack, hp_min = 1;
    int hp;
    if(attack > 0 && !need_atk) continue;
    if(!has_hp)
      hp_max = hp_min = 0;
    for(hp = hp_min; hp <= hp_max; hp++)
    {
      Kenshi x;
      int defense = bonus-attack-hp;
      ken_init(&x, 5*hp, attack, defense, spd, swords);
      print_kenshi(&x);
    }
  }
}

int main(void)
{
  int bonus;
  kenshi_set set;
  Kenshi ks[5];
  ken_read_theki(&ks[0], "15/0/1/2/速熱衝絶/徘徊するライカ");
  ken_read_theki(&ks[1], "20/0/0/1/速回4斬命死/Cat_Panch_Sword");
  ken_read_theki(&ks[2], "10/2/0/3/熱斬斬/●野　啓");
  ken_read_theki(&ks[3], "5/3/0/3/毒毒毒/掌");
  ken_read_theki(&ks[4], "5/0/0/4/速熱熱衝衝/ヤマハ");

  set.num = sizeof ks / sizeof *ks;
  set.ks = ks;
  for(bonus = 0; bonus <= 10; bonus++)
    search_with_bonus(bonus, &set);
  /*
  int has_hp;

  for(has_hp = 0; has_hp <= 1; has_hp++)
  {
    int spd;
    for(spd = 0; spd <= 6; spd++)
      each_sword_array(20, spd, has_hp, 6-spd, 6-spd, enum_all, 0);
  }
  */
  
  /*/
  ken_each_in(6, print_kenshi, 20);
  /**/
  printf("g_counter = %d\n", g_counter);
  return 0;
}
